CACHE_MINUTE = 60
CACHE_15_MINS = 900
CACHE_30_MINS = 1800
CACHE_HOUR = 3600
CACHE_8_HOURS = 28800
CACHE_12_HOURS = 43200
CACHE_DAY = 86400
CACHE_2_DAYS = 172800
CACHE_WEEK = 604800
CACHE_2_WEEKS = 1209600
CACHE_MONTH = 2592000
